#include <windows.h>
#include <iostream>
#include <vector>
#include <algorithm>

using namespace std;

union FTime {
	FILETIME ft;
	long long lv;
};

const int COUNT = 1000000;
vector<int> vec;

int RandomNumber () {
	static int s = 0;
	return (s++ % 100);
}


typedef void (*BmProc)(void);

#define BM_FOREACH_LAMBDA
void bm_foreach_lambda()
{
	int sum = 0;
	for_each(vec.begin(), vec.end(), [&sum](int i) {
		sum += i;
	});
}

#define BM_VEC_ITERS
void bm_vec_iters()
{
	auto end = vec.end(); int sum = 0;
	for(auto i = vec.begin(); i != end; ++i)
		sum += *i;
}

#define BM_VEC_NO_ITERS
void bm_vec_no_iters()
{
	int sum = 0;
	for(int i = 0; i < COUNT; ++i)
		sum += vec[i];
}

#define BM_ARRAY_FROM_VEC
void bm_array_from_vec()
{
	int sum = 0;
	int *array = &vec[0];
	for(int i = 0; i < COUNT; ++i) {
		sum += array[i];
	};
}

#define BM_ITERATE_BY_PTR
void bm_iterate_by_ptr()
{
	int sum = 0;
	auto end = &vec[vec.size () - 1];
	for(int *i = &vec[0]; i != end; ++i) {
		sum += *i;
	};
}

void RunOne(BmProc prc, const char *name)
{
	FTime t0, t1;
	GetSystemTimeAsFileTime(&t0.ft);
	for (int j = 0; j < 100; j++)
		(*prc)();
	GetSystemTimeAsFileTime(&t1.ft);
	std::cout << name << " t=" << (t1.lv-t0.lv)/10 << " mcs" << std::endl;
}

int main () 
{
	vec.resize(COUNT);
	generate(vec.begin(), vec.end(), RandomNumber);
#ifdef BM_FOREACH_LAMBDA
	RunOne(bm_foreach_lambda, "Foreach lambda");
#endif
#ifdef BM_VEC_ITERS
	RunOne(bm_vec_iters, "Vector Iterators");
#endif
#ifdef BM_VEC_NO_ITERS
	RunOne(bm_vec_no_iters, "Vector No Iterators");
#endif
#ifdef BM_ARRAY_FROM_VEC
	RunOne(bm_array_from_vec, "Array from Vector");
#endif
#ifdef BM_ITERATE_BY_PTR
	RunOne(bm_iterate_by_ptr, "Iterate by Ptr");
#endif
	return 0;
};